/*********************************************************************************************

	main.c

	DTMF decorder

	CPU:				dsPIC30F2020(use +5V)
	DTMF DECORD Chip    CM8870

	CPU Clock
		internal FRC_PLL, Tcy=29MHz

	CPU I/O Pin ASSIGN

	                 MCR#(1)  (28)AVDD -------- +5
	                  RB0(2)  (27)AVSS -------- GND
	                  RB1(3)  (26)RE0 --------- Q1
	                  RB2(4)  (25)RE1 --------- Q2
	                  RB3(5)  (24)RE2 --------- Q3
	                  RB4(6)  (23)RE3 --------- Q4
	                  RB5(7)  (22)RE4 --------- StD
	    GND --------- VSS(8)  (21)RE5 --------- Power LED
	             OSC1/RB6(9)  (20)VDD --------- +5V
                 OSC2/RB7(10) (19)VSS --------- GND
  UART-TX ----- U1ATX/RE7(11) (18)RF7/SDA ----- OLED-SDA
  UART-RX ----- U1ARX/RE6(12) (17)RF8/SCL ----- OLED-SCL
        +5V --------- VDD(13) (16)RA9
 PICKit2-PGD --- PGD2/RF6(14) (15)RD0/PGC2 ---- PICKit2-PGC

*********************************************************************************************/

#include <string.h>
#include <p30f2020.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "timer.h"
#include "oled.h"

/*=====================================================================================================
	Device Configuration
=====================================================================================================*/

_FBS(BWRP_OFF & NO_BOOT_CODE)
_FGS(GWRP_OFF & CODE_PROT_OFF)
_FOSCSEL(FRC_PLL)
_FOSC(OSC2_IO & FRC_HI_RANGE & CSW_FSCM_OFF)
_FWDT(FWDTEN_OFF)
_FPOR(PWRT_128)
_FICD(ICS_PGD2)


/*=====================================================================================================
	Definition
=====================================================================================================*/


/*=====================================================================================================
	Global Variable
=====================================================================================================*/


/*=====================================================================================================
	function prototype
=====================================================================================================*/

void io_initialize(void);


/*=====================================================================================================
	main
=====================================================================================================*/

int	main(void)
{
    int val, tid;
    int len, current;
    char dial[2][32];
    char *digitnum = "D1234567890.#ABCD";

	// system initialize
	io_initialize();
    rs_initialize();
    timer_init();
    rs_putstr("\r\n*** Start DTMF decorder **********\r\n");
    oled_init();
    rs_putstr("complete oled init.\r\n");
    oled_putstr2(0,0,"--DTMF Decord--");

    memset(dial[0], ' ', 16); dial[0][16]=0;
    strcpy(dial[1],"--DTMF Decord-- "); 
    current = 0;
    len = 0;

    tid = timer_entry(1000);
    for(;;){
        // wait untile bit4 up
        val = PORTE;
        if((val & 0x10)==0){          // bit4 off
            if(timer_check(tid) ==0){
                if(len){
                    //dial[len]=0;
                    rs_putstr("dial="); rs_putstr(dial[current]); rs_putstr("\r\n");

                    oled_putstr2(0,0, dial[current^1]);
                    oled_putstr2(0,2, dial[current]); 
                    current ^= 1;
                    memset(dial[current], ' ', 16); dial[current][16]=0;
                    len=0;
                }
            }
            continue;
        }

        // store dial number, max length 16chars.
        if(len < 16){
            dial[current][len] = digitnum[val & 0xF];
            len++;
        }
        while(PORTE & 0x10);  // wait until bit4 down
        timer_update(tid, 200);
    }
	return 0;
}


/*-----------------------------------------------------------------------------------------------------
	initialize 
-----------------------------------------------------------------------------------------------------*/

void	io_initialize(void)
{
	//--- setting interrupt ---------------------------------------

	//--- setting I/O port ----------------------------------------
	ADPCFG = 0b1111111111111111;// Analog pin all disable
    TRISA = 0xffff;
	TRISB = 0xffff;
	TRISD = 0xffff;
    TRISE = 0xffff;
    TRISF = 0xffff;
    LATA = 0;
	LATB = 0;
	LATD = 0;
    LATE = 0;
    LATF = 0xffff;

    TRISEbits.TRISE7 = 0;  // RE7(U1ATX) output
    LATEbits.LATE7   = 1;  // output level high
   
    TRISEbits.TRISE5 = 0;  // power LED
    LATEbits.LATE5 = 0;    //  LED on

    // I2C on
    I2CCON = 0;
    // sample dsPIC30F3012
    //I2CBRG = 93;
    // Fsc=400KHZ, Fcy=7.37x8=58.96MHz,
	// (58.96MHZ/400KHZ-58.96MHZ/1.111111MHZ)-1=147.4-53.06-1=94.34-1=93
    //I2CBRG = (58000000/400000 - 58000000/1111111)-1;
    //I2CBRG = (145 - 52.2)-1=92;    
    I2CBRG = 92;

}



/*=====================================================================================================
=====================================================================================================*/

/*=====================================================================================================
=====================================================================================================*/

/*=====================================================================================================
=====================================================================================================*/

/*=====================================================================================================
=====================================================================================================*/

/*=====================================================================================================
=====================================================================================================*/


/*=====================================================================================================
=====================================================================================================*/




/*=====================================================================================================
=====================================================================================================*/

/*-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------*/


/*** end of main.c ******************************************************************/
