/*********************************************************************

	timer.c
	
	application timer module

**********************************************************************/

#include	<p30F2020.h>
#include	"timer.h"


/*** global variable **************************************************/

// for application timer
static	volatile unsigned int tm[10];
static	volatile unsigned char  tuse[10];

/*-----------------------------------------------------------------------
	initilaize timer system
-----------------------------------------------------------------------*/

void timer_init(void)
{
	int	i;

	// clear timer table
	for(i = 0; i < 10; i++) {
		tm[i] = tuse[i] = 0;
	}


	//--- setting interrupt ---------------------------------------
	INTCON1 = 0;		// NSTDIS=0 nesting ok
	IEC0bits.T2IE = 1;	// enable T2 interrupt

	//--- set interval@1ms timer --------------------------------
	T2CON = 0x0000;	// use timer(T2), set prescale 1:1
	TMR2 = 0;
	PR2 = 28999;		// 1msec (29,000,000Hz/1000Hz=29000)

	T2CONbits.TON = 1;	// start T2


}


/*----------------------------------------------------------------------
	Timer T2 interrupt
	Timer T2 1msec interval timer.
----------------------------------------------------------------------*/

void __attribute__ ((__interrupt__, no_auto_psv)) _T2Interrupt(void)
{
	int	i;

	IFS0bits.T2IF = 0;		// clear interrupt flag

	// timer count down
	for(i = 0; i < 10; i++){
		if(tm[i]) tm[i]--;
	}
}


/*----------------------------------------------------------------------
	timer_entry()
----------------------------------------------------------------------*/

int	timer_entry(unsigned int tm_val)
{
	int n;

	for(n = 0; n < 10; n++){
		if(tuse[n] == 0){ 
			tuse[n] = 1;
			asm("DISI #100");
			tm[n] = tm_val;
			asm("DISI #0");
			return n;
		}
	}
	return -1;
}


/*----------------------------------------------------------------------
	timer_update()
----------------------------------------------------------------------*/

void timer_update(int n, unsigned int tm_val)
{
	if(n >= 0 && n < 10){
		IEC0bits.T2IE=0;
		tm[n] = tm_val;
		IEC0bits.T2IE=1;
	}
}


/*----------------------------------------------------------------------
	timer_delete()
----------------------------------------------------------------------*/

void timer_delete(int n)
{
	if(n >= 0 && n < 10){
		tuse[n] = 0;
	}
}


/*----------------------------------------------------------------------
	timer_check()
----------------------------------------------------------------------*/

int timer_check(int n)
{
	if(n >= 0 && n < 10){
		return tm[n];
	}
	return 0;
}


/*----------------------------------------------------------------------
	timer_Up()
----------------------------------------------------------------------*/

int timer_Up(int n)
{
	if(n >= 0 && n < 10){
		return tm[n] == 0 ? 1: 0;
	}
	return 0;
}


/*** end of tskchg.c *************************************************/

